function list(path, remember, skipCache, deletingRecursively) {
	if (!skipCache) {
		try {																					// check the cache first
			var cacheService = Components.classes["@mozilla.org/network/cache-service;1"]
				.getService(Components.interfaces.nsICacheService);
			var cacheSession = cacheService.createSession("fireftp", 0, true);
			var cacheDesc = cacheSession.openCacheEntry("ftp://" + version + connectedHost + path,
				Components.interfaces.nsICache.ACCESS_READ, false);
			if (cacheDesc.dataSize) {
				var cacheIn = cacheDesc.openInputStream(0);
				var cacheInstream = Components.classes["@mozilla.org/binaryinputstream;1"]
					.createInstance(Components.interfaces.nsIBinaryInputStream);
				cacheInstream.setInputStream(cacheIn);
				data = cacheInstream.readBytes(cacheInstream.available());
				data = eval(data);
				cacheInstream.close();
				cacheDesc.close();
				eval(remember);
				return;
			}
			cacheDesc.close();
		} catch (ex) { }
	}

	if (deletingRecursively) {																// do things backwards if deleting
		eventQueue.unshift( { cmd: "LIST",	parameter: (hiddenmode ? "-al " : '') + path, 	remember: remember } );
		eventQueue.unshift( { cmd: "PASV",	parameter: "",									remember: "" } );
		eventQueue.unshift( { cmd: "CWD",	parameter: path,								remember: "" } );
		eventQueue.unshift( { cmd: "TYPE",	parameter: "A",									remember: "" } );
	} else {
		addEventQueue("TYPE",	"A");
		addEventQueue("CWD",	path);
		addEventQueue("PASV");
		addEventQueue("LIST",	(hiddenmode ? "-al " : '') + path,	remember);
	}
	if (isReady)
		writeControl(eventQueue[0].cmd, eventQueue[0].parameter);
}

// *************************************** parseListData ****************************************

function parseListData(thedata, path) {
	/* Unix style:                     drwxr-xr-x 1 user01 ftp 512 Jan 29 23:32 prog
	 * Alternate Unix style:           drwxr-xr-x 1 user01 ftp 512 Jan 29 1997  prog
	 * Alternate Unix style:           drwxr-xr-x 1 1      1   512 Jan 29 23:32 prog
	 * A symbolic link in Unix style:  lrwxr-xr-x 1 user01 ftp 512 Jan 29 23:32 prog -> prog2000
	 * MS-DOS style:                   01-29-97 11:32PM <DIR> prog
	 * Novell Style:                   drwxr-xr-x 1 user01     512 Jan 29 23:32 prog
	 * OS/2 style:                     0           DIR 01-29-97  23:32  PROG
	 * OS/2 style:                     2243        RA  04-05-103 00:22  PJL
	 * OS/2 style:                     60              11-18-104 06:54  chkdsk.log
	 * AIX style:                      -rw-r--r-- 1 sys   sys   30 05 Nov 2003  foo.txt
	 *
	 * CURRENTLY NOT SUPPORTED: (are these even used anymore?  i can't find any ftp servers using these)
	 *  Macintosh style:       drwxr-xr-x folder 0 Jan 29 23:32 prog
	 */

	debug(thedata, 0, "INFO ");

	var treeitem = thedata.split("\r\n");

	// some ftp servers send 'count <number>' or 'total <number>' first
	if (treeitem.length && (treeitem[0].indexOf("count") == 0 || treeitem[0].indexOf("total") == 0))
		treeitem.shift();

	for (var x = 0; x < treeitem.length; ++x) {
		if (!treeitem[x]) { // some servers put in blank lines b/w entries, aw, for cryin' out loud
			treeitem.splice(x, 1);
			--x;
			continue;
		}
		var temp = treeitem[x];
		treeitem[x] = treeitem[x].split(" ");
		if (!parseInt(treeitem[x][0].charAt(0)) && treeitem[x][0].charAt(0) != '0')  {	// unix style - so much simpler with you guys
			var offset = 0;
			if (treeitem[x][4].search(remoteMonths) != -1)								// added to support novell servers
				offset = 1;
			var name = temp.substring(temp.indexOf(treeitem[x][7 - offset]) + treeitem[x][7 - offset].length + 1, temp.length);
			var symlink = "";
			name = name.substring(name.search(/[^\s]/));
			if (treeitem[x][0].charAt(0) == 'l') {
				symlink = name;
				name = name.substring(0, name.indexOf("->") - 1);
				symlink = symlink.substring(symlink.indexOf("->") + 3);
			}
			treeitem[x] = {
				permissions: treeitem[x][0],
				hardLink: treeitem[x][1],
				user: treeitem[x][2],
				group: (offset ? "" : treeitem[x][3]),
				fileSize: treeitem[x][4 - offset],
				date: treeitem[x][5 - offset] + ' ' + treeitem[x][6 - offset] + ' ' + treeitem[x][7 - offset],
				leafName: (name.lastIndexOf('/') == -1 ? name : name.substring(name.lastIndexOf('/') + 1)),
				isDirectory: treeitem[x][0].charAt(0) == 'd',
				isSymlink: treeitem[x][0].charAt(0) == 'l',
				symlink: symlink
			};
		} else if (treeitem[x][0].indexOf('-') == -1) {									// os/2 style
			var offset = 0;
			if (treeitem[x][2].indexOf(':') != -1)										// if "DIR" and "A" are missing
				offset = 1;
			var rawDate = treeitem[x][2 - offset].split("-");
			var rawTime = treeitem[x][3 - offset];
			var timeOrYear = rawTime;
			rawTime = rawTime.split(":");
			for (var y = 0; y < rawDate.length; ++y) {
				if (rawDate[y].charAt(0) == '0')										// javascript, seriously - can't you parse 09 to 9 and not 0?
					rawDate[y] = rawDate[y][1];
				rawDate[y] = parseInt(rawDate[y]);
			}
			for (var y = 0; y < rawTime.length; ++y) {
				if (rawTime[y].charAt(0) == '0')
					rawTime[y] = rawTime[y][1];
				rawTime[y] = parseInt(rawTime[y]);
			}
			rawDate[2] = rawDate[2] + 1900;												// ah, that's better
			parsedDate = new Date(rawDate[2], rawDate[0] - 1, rawDate[1], rawTime[0], rawTime[1]);	// month-day-year format
			if (new Date() - parsedDate > 15600000000)									// roughly 6 months
				timeOrYear = rawDate[2];
			var month = parsedDate.toLocaleDateString().split(" ");
			var name = temp.substring(temp.indexOf(treeitem[x][3 - offset]) + treeitem[x][3 - offset].length + 1, temp.length);
			name = name.substring(name.search(/[^\s]/));
			month = month[1].substring(0,3);	// abbr.
			treeitem[x] = {
				permissions: treeitem[x][1] == "DIR" ? "d---------" : "----------",
				hardLink: "",
				user: "",
				group: "",
				fileSize: treeitem[x][0],
				date: month + ' ' + rawDate[1] + ' ' + timeOrYear,
				leafName: (name.lastIndexOf('/') == -1 ? name : name.substring(name.lastIndexOf('/') + 1)),
				isDirectory:  treeitem[x][1] == "DIR",
				isSymlink: false,
				symlink: ""
			};
		} else {																		// ms-dos style
			var rawDate = treeitem[x][0].split("-");
			var rawTime = treeitem[x][1].substring(0, 5);								// get rid of PM, AM
			var timeOrYear = rawTime;
			rawTime = rawTime.split(":");
			for (var y = 0; y < rawDate.length; ++y) {
				if (rawDate[y].charAt(0) == '0')										// javascript, seriously - can't you parse 09 to 9 and not 0?
					rawDate[y] = rawDate[y][1];
				rawDate[y] = parseInt(rawDate[y]);
			}
			for (var y = 0; y < rawTime.length; ++y) {
				if (rawTime[y].charAt(0) == '0')
					rawTime[y] = rawTime[y][1];
				rawTime[y] = parseInt(rawTime[y]);
			}
			if (rawDate[2] < 70)														// assuming you didn't have some files left over from 1904
				rawDate[2] = rawDate[2] + 2000;											// ah, that's better
			parsedDate = new Date(rawDate[2], rawDate[0] - 1, rawDate[1], rawTime[0], rawTime[1]);	// month-day-year format
			if (new Date() - parsedDate > 15600000000)									// roughly 6 months
				timeOrYear = rawDate[2];
			var month = parsedDate.toLocaleDateString().split(" ");
			var name = temp.substring(temp.indexOf(treeitem[x][2], temp.indexOf(treeitem[x][1]) + treeitem[x][1].length + 1) + treeitem[x][2].length + 1, temp.length);
			name = name.substring(name.search(/[^\s]/));
			month = month[1].substring(0,3);											// abbr.
			treeitem[x] = { 
				permissions: treeitem[x][2] == "<DIR>" ? "d---------" : "----------",
				hardLink: "",
				user: "",
				group: "",
				fileSize: treeitem[x][2] == "<DIR>" ? '0' : treeitem[x][2],
				date: month + ' ' + rawDate[1] + ' ' + timeOrYear,
				leafName: (name.lastIndexOf('/') == -1 ? name : name.substring(name.lastIndexOf('/') + 1)),
				isDirectory: treeitem[x][2] == "<DIR>",
				isSymlink: false,
				symlink: ""
			};
		}
	}

	var directories = new Array;														// sort directories to the top
	var files = new Array;
	for (var x = 0; x < treeitem.length; ++x) {
		if (treeitem[x].isDirectory)
			directories.push(treeitem[x]);
		else
			files.push(treeitem[x]);
	}

	if (directories.length && directories[0].leafName == ".")							// get rid of "." or ".."
		directories.shift();															// this can screw up things on recursive deletions
	if (directories.length && directories[0].leafName == "..")
		directories.shift();
	
	treeitem = directories.concat(files);

	if (sessionsmode) {
		try {																			// put in cache
			var cacheService = Components.classes["@mozilla.org/network/cache-service;1"]
				.getService(Components.interfaces.nsICacheService);
			var cacheSession = cacheService.createSession("fireftp", 0, true);
			var cacheDesc = cacheSession.openCacheEntry("ftp://" + version + connectedHost + path,
				Components.interfaces.nsICache.ACCESS_WRITE, false);
			var cacheOut = cacheDesc.openOutputStream(0);
			var cacheData = treeitem.toSource();
			cacheOut.write(cacheData, cacheData.length);
			cacheOut.close();
			cacheDesc.close();
		} catch (ex) { debug(ex,22); }
	}

	return treeitem;
}
function checkIsListing() {																// check queue to see if we're listing
	for (var x = 0; x < eventQueue.length; ++x)
		if (eventQueue[x].cmd.indexOf("LIST") != -1)
			return true;
	return false;
}
