var localDirTree = {
	data : new Array,
    rowCount : 0,
    getCellText : function(row,column) {		// names for the directories
		return row == -1 ? "" : this.data[row].leafName;
    },
	getImageSrc : function(row,col) {			// icons for the directories
		return this.isContainerOpen(row) ? "res/directoryopen.ico" : "res/directory.ico";
	},
	getLevel : function(row) {					// level is based on number of slashes in the path
		if (row <= 0 || row >= this.data.length)
			return 0; 
		return this.data[row].path().match(slash == "/" ? /\x2f/g : /\x5c/g ).length;
	},
	getParentIndex : function(row) {
		if (row <= 0)
			return -1;
		return this.indexOfPath(this.data[row].parent);
	},
	getCellProperties	: function(row,col,props)	{},
    getColumnProperties	: function(colid,col,props)	{},
	getRowProperties	: function(row,props)		{},
 	hasNextSibling		: function(row,nextrow)		{ return this.data[row].hasNext; },
  	isContainer			: function(row)				{ return true; },
 	isContainerEmpty	: function(row)				{ return this.data[row].empty; },
	isContainerOpen		: function(row)				{ return this.data[row].open; },
	isSeparator			: function(row)				{ return false; },
    isSorted			: function(row)				{ return false; },
    setTree				: function(treebox)			{ this.treebox = treebox; },
	toggleOpenState		: function(row)				{
		if (this.isContainerOpen(row)) {
			var currentIndex = this.selection.count ? this.selection.currentIndex : -1;
			
			var level = this.getLevel(row);
			var lastChild = row;
			while (lastChild + 1 < this.rowCount && this.getLevel(lastChild + 1) > level)		// find last index in same level as collapsed dir	
				++lastChild;
			this.data[row].children = this.data.splice(row + 1, lastChild - row);				// get rid of subdirectories from view
			this.rowCount = this.data.length;
			this.treebox.rowCountChanged(row, -(lastChild - row));

			this.data[row].open = false;
			this.treebox.invalidateRow(row);													// update row

			if (gLocalPath.value.indexOf(this.data[row].path()) != -1
					&& gLocalPath.value != this.data[row].path()) {								// we were in a subdirectory and we collapsed
				gLocalPath.value = this.data[row].path();
				localTree.updateView();
				this.selection.select(row);
				this.treebox.ensureRowIsVisible(row);
			} else if (gLocalPath.value == this.data[row].path()) {
				this.selection.select(row);
				this.treebox.ensureRowIsVisible(row);
			} else if (currentIndex != -1 && currentIndex < this.rowCount)
				this.selection.select(currentIndex);
		} else {
			if (this.data[row].children) {														// stored from before
				for (var x = this.data[row].children.length - 1; x >= 0; --x)
					this.data.splice(row + 1, 0, this.data[row].children[x]);
				this.rowCount = this.data.length;
				this.treebox.rowCountChanged(row + 1, this.data[row].children.length);
				this.data[row].children = null;
				this.data[row].open = true;
				this.treebox.invalidateRow(row);
			} else {																			// get data for this directory
				var newDirectories = new Array;
				
				try {
					var dir = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
					dir.initWithPath(this.data[row].path());
					var entries = dir.directoryEntries;
					while (entries.hasMoreElements()) {
						var file = entries.getNext().QueryInterface(Components.interfaces.nsILocalFile);	// get subdirectories
						if (file.exists() && file.isDirectory() && (!file.isHidden() || hiddenmode))
							newDirectories.push(file);
					}
				} catch (ex) {
					error(strbundle.getString("noPermission"));
					debug(ex,31);
				}

				if (newDirectories.length == 0)	{												// no subdirectories
					this.data[row].empty = true;
					this.data[row].open = false;
				} else {																		// has subdirectories
					for (var x = 0; x < newDirectories.length; ++x)
						newDirectories[x] = { leafName: newDirectories[x].leafName, parent: newDirectories[x].parent ? newDirectories[x].parent.path: "",
											  path: function() { return localTree.constructPath(this.parent, this.leafName); },
											  open: false, empty: false, children: null, hasNext: true };

					newDirectories.sort(directorySort);
					newDirectories[newDirectories.length - 1].hasNext = false;	// last one doesn't have a next
					for (var x = newDirectories.length - 1; x >= 0; --x)
						this.data.splice(row + 1, 0, newDirectories[x]);
					this.rowCount = this.data.length;
					this.treebox.rowCountChanged(row + 1, newDirectories.length);
					this.data[row].open = true;
				}
				this.treebox.invalidateRow(row);
			}
		}
	},

	// ************************************************* mouseEvent *****************************************************

	click : function(event) {
		if (event.button == 0 || event.button == 2) {
			var row = {};		var col = {};		var child = {};
			this.treebox.getCellAt(event.pageX, event.pageY, row, col, child);
			var index = this.selection.currentIndex;

			if (index == row.value && this.data[index].path() != gLocalPath.value)				// index == row.value in case were collapsing the folder
				this.changeDir(this.data[index].path());
		} else if (event.button == 1 && !document.getElementById('localPasteContext').disabled)	// middle-click paste
			this.paste();
	},

	// ************************************************* keyEvent *****************************************************

	localDirKeyDownIndex: -1,
	localDirKeyDownOpen	: -1,
	keyDown  : function(event) {
		this.localDirKeyDownIndex = this.selection.currentIndex;
		this.localDirKeyDownOpen = this.data[this.localDirKeyDownIndex].open;
	},
	keyPress : function(event) {
		var index = this.selection.currentIndex;
		switch (event.keyCode) {
			case 37:										// left arrow
				if (!this.data[index].open && this.getParentIndex(index) != -1) {
					gLocalPath.value = this.data[this.getParentIndex(index)].path();
					localTree.updateView();
					this.selection.select(index);
				}
				break;
			case 38:										// up arrow
			case 40:										// down arrow
				if (event.keyCode == 38 && !index)
					return;
				if (event.keyCode == 40 && index == this.data.length - 1) {
					this.selection.select(index - 1);		// this shouldn't work but it does
					return;
				}
				var x = event.keyCode == 38 ? -1 : 1;

				// see if file is good first
				try {
					dir = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
					dir.initWithPath(this.data[index + x].path());
					var entries = dir.directoryEntries;
				} catch (ex) {
					debug(ex,28);
					this.changeDir(this.data[index + x].path());
					this.selection.select(index);
					break;
				}

				gLocalPath.value = this.data[index + x].path();
				localTree.updateView();
				this.selection.select(index);
				break;
			case 39:										// right arrow
				if (this.localDirKeyDownOpen && !this.data[index].empty) {

					// see if file is good first
					try {
						dir = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
						dir.initWithPath(this.data[index + 1].path());
						var entries = dir.directoryEntries;
					} catch (ex) {
						error(strbundle.getString("noPermission"));
						debug(ex,30);
						break;
					}

					gLocalPath.value = this.data[index + 1].path();
					localTree.updateView();
				}
				break;
			case 8:											// backspace
				this.cdup();
				break;
			case 116: 										// F5
				event.preventDefault();
				localTree.refresh();
				break;
		}
	},

	// ********************************************** file functions ***************************************************

	getFile : function(row) {		// get file based on path
		var file = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
		file.initWithPath(this.data[row].path());
		return file;
	},
	indexOfPath : function(path) {	// binary search to find a path in the localDirTree
		var left = 0;
		var right = this.data.length - 1;
		path = path.toLowerCase();

		while (left <= right) {
			var mid = Math.floor((left + right) / 2);
			var dataPath = this.data[mid].path().toLowerCase();
			if (dataPath == path || dataPath + slash == path || dataPath == path + slash)	// kind of complicated but what can you do
				return mid;
			else if (path < dataPath)
				right = mid - 1;
			else if (path > dataPath)
				left = mid + 1;
		}
		return -1;
	},
	reselectCurrentDirectory : function() {
		var index = this.indexOfPath(gLocalPath.value);								// reselect the current directory
		this.selection.select(index);
		this.treebox.ensureRowIsVisible(index);
	},
						
   	// ************************************************* cdup *****************************************************

	cdup : function() {
		var parentIndex = this.getParentIndex(this.selection.currentIndex);
		if (parentIndex != -1) {
			this.changeDir(this.data[parentIndex].path());
			this.selection.select(parentIndex);
		}
	},

	// ************************************************* change directories *****************************************************

	changeDir : function (path) {
		gLocalPath.value = path;
		if (localDirTree.data.length == 0 || localDirTree.data[0].path().charAt(0) != gLocalPath.value.charAt(0)) {		// if dirTree is empty
			var oldRowCount = localDirTree.rowCount;																	// or we're switching to a new drive
			localDirTree.data = new Array;
			localDirTree.rowCount = 0;
			localDirTree.treebox.rowCountChanged(0, -oldRowCount);

			var thepath;
			if (gLocalPath.value.indexOf('/') == 0) { 	// linux														// we restart the tree
				thepath = "/";
				slash = "/";
			} else {									// windows
				if (gLocalPath.value.indexOf('\\') == -1) {
					gLocalPath.value += "\\";
				}
				thepath = gLocalPath.value.substring(0, gLocalPath.value.indexOf('\\') + 1);
				slash = "\\";
			}

			localDirTree.data.push( {leafName: thepath, parent: "",
									 path: function() { return this.leafName; },
									 open: false, empty: false, children: null, hasNext:false} );
			localDirTree.rowCount = 1;
			localDirTree.treebox.rowCountChanged(0, 1);
			localDirTree.selection.select(0);
		}

		if (slash == "/")												// error checking here for correct values in path
			gLocalPath.value = gLocalPath.value.replace(/\x5c/g, "/");	// linux shouldn't have backslashes
		else
			gLocalPath.value = gLocalPath.value.replace(/\x2f/g, "\\");	// windows shouldn't have forward slashes

		if (gLocalPath.value != '/' && gLocalPath.value.charAt(gLocalPath.value.length - 1) == slash)	// trim slashes at the end - ruins levels in dir
			gLocalPath.value = gLocalPath.value.substring(0, gLocalPath.value.length - 1);
		if (slash == "\\" && gLocalPath.value.indexOf('\\') == -1)										// if it's windows we add a slash to it: 'c:'->'c:\'
			gLocalPath.value += "\\";
		if (slash == "/" && gLocalPath.value.charAt(0) != '/')	// linux path has to start with '/'
			gLocalPath.value = '/' + gLocalPath.value;

		var index = this.indexOfPath(gLocalPath.value);											// find directory in localDirTree if it's there
		if (index != -1) {
			if (!this.data[index].open)
				this.toggleOpenState(index);
			localPathFocus = path;																// directory approved
			var gFormHistory = Components.classes["@mozilla.org/satchel/form-history;1"]		// add directory to drop down list
				.getService(Components.interfaces.nsIFormHistory);
			gFormHistory.addEntry(gLocalPath.getAttribute("autocompletesearchparam"), gLocalPath.value);

			savePreferences();																	// remember last directory

			localTree.updateView();
			return;
		}

		for (var x = 0; x < localDirTree.data.length; ++x)										// else we have to open parent directories til we find it
			if (gLocalPath.value.indexOf(localDirTree.data[x].path()) != -1) {
				if (!localDirTree.data[x].open)
					localDirTree.toggleOpenState(x);
				if (gLocalPath.value == localDirTree.data[x].path()) {
					localPathFocus = path;														// directory approved
					var gFormHistory = Components.classes["@mozilla.org/satchel/form-history;1"]// add directory to drop down list
							.getService(Components.interfaces.nsIFormHistory);
					gFormHistory.addEntry(gLocalPath.getAttribute("autocompletesearchparam"), gLocalPath.value);

					savePreferences();															// remember last directory

					localTree.updateView();
					return;
				}
			}
	},
};
